# -*- coding: utf-8 -*-
#################################################################################
#    Autor: Roberto Lizana (robertolizana@trey.es)
#    Copyright (C) 2010 Trey (<http://www.trey.es>). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#################################################################################
from osv import osv, fields
from tools import config
from tools.translate import _
import decimal_precision as dp
import netsvc
import pooler
import time

def create_slug(string) :
    def multiple_replace(dic, text): 
        import re 
        pattern = "|".join(map(re.escape, dic.keys()))
        return re.sub(pattern, lambda m: dic[m.group()], text) 

    rplc = {' ' : '_', 'á' : 'a', 'é' : 'e', 'í' : 'i', 'ó' : 'o', 'ú' : 'u', 'ç' : 'c', 'ñ' : 'n'}
    name = name.strip().lower()
    name = multiple_replace(rplc,string)
    import re
    return re.sub('/[^a-z_0-9]/', '', string)        

class ledia_product_color(osv.osv):
    _name = "ledia.product.color"
    _description = "Color de producto"
    _columns = {
        'name': fields.char('Nombre', size=255, required=True, translate=True),
        'description_sale': fields.text('Descripción de venta',translate=True),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_color()

class ledia_product_color_temp(osv.osv):
    _name = "ledia.product.color_temp"
    _description = "Temperatura del color"
    _columns = {
        'name': fields.char('Nombre', size=255, required=True, translate=True),
        'description_sale': fields.text('Descripción de venta',translate=True),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_color_temp()

class ledia_product_lamp_size(osv.osv):
    _name = "ledia.product.lamp_size"
    _description = "Tamaño de lampara"
    _columns = {
        'name': fields.char('Nombre', size=255, required=True),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_lamp_size()

class ledia_product_format(osv.osv):
    _name = "ledia.product.format"
    _description = "Formato del producto"
    _columns = {
        'name': fields.char('Nombre', size=255, required=True),
        'description_sale': fields.text('Descripción de venta',translate=True),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_format()

class ledia_product_cert(osv.osv):
    _name = "ledia.product.cert"
    _description = "Certificado del producto"
    _columns = {
        'name': fields.char('Nombre', size=255, required=True),
        'image_filter': fields.char('Filtro de imagen',size=255),
        'description_sale': fields.text('Descripción de venta',translate=True),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_cert()

class ledia_product_application(osv.osv):
    _name = "ledia.product.application"
    _description = "Aplicación del producto"
    _columns = {
        'name': fields.char('Nombre', size=255),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_application()

class ledia_product_lens_type(osv.osv):
    _name = "ledia.product.lens_type"
    _description = "Tipo de lente LED"
    _columns = {
        'name': fields.char('Nombre', size=255),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_lens_type()

class ledia_product_lampholder(osv.osv):
    _name = "ledia.product.lampholder"
    _description = "Casquillo"
    _columns = {
        'name': fields.char('Nombre', size=255),
    }
    _order = 'name asc'
    _sql_constraints = []
ledia_product_lampholder()

### INHERITED ###
class product_product(osv.osv):

    def calc_luminous_efficacy(self, cr, uid, ids, field_name, attr, context=None):
        res = {}
        for pr in self.browse(cr, uid, ids, context=context):
            if pr.prop_luminous_flux and pr.prop_power:
                res[pr.id] = pr.prop_luminous_flux / pr.prop_power
            else :
                res[pr.id] = 0
        return res
    

    def calc_power_saving(self, cr, uid, ids, field_name, attr, context=None):
        res = {}
        for pr in self.browse(cr, uid, ids, context=context):
            if pr.prop_power_replace and pr.prop_power:
                res[pr.id] = pr.prop_power_replace / pr.prop_power
            else :
                res[pr.id] = 0
        return res
            
    _inherit = 'product.product'
    _columns = {
        'logistic_code': fields.char('Cód. logistico',size=255),
        'prop_color': fields.many2one('ledia.product.color','Color'),
        'prop_color_temp': fields.many2one('ledia.product.color_temp','Temp. color (ºK)'),
        'prop_lamp_size': fields.many2one('ledia.product.lamp_size','Tam. lamp. (mm)', help="Tamaño de lampara"),
        'prop_lamp_diameter': fields.integer('Ø lamp. (mm)', help="Diametro de lampara"),
        'prop_power_consumption': fields.integer('Consumo de enegría'),
        'prop_lifetime': fields.selection([('20.000','20.000'),('28.000','28.000'),('30.000','30.000'),('40.000','40.000'),('80.000','80.000')],'Vida útil (horas)'),
        'prop_service_temp': fields.float('Temp. servicio (ºC)'),
        'prop_use_temp': fields.float('Temp. de uso (ºC)'),
        'prop_direct_tension': fields.integer('Tensión directa REC(V)'),
        'prop_direct_tension_range': fields.char('Tensión directa RANGO(V)', size=255),
        'prop_current_direct': fields.integer('CC REC(Ma)', help="Corriente directa o continua"),
        'prop_current_direct_range': fields.char('CC RANGO(V)', size=255, help="Corriente directa o continua"),
        'prop_lens_type': fields.many2one('ledia.product.lens_type','Tipo de lente LED'),
        'prop_manufacturer_led_chip': fields.many2one('res.partner','Fabricante chip'),
        'prop_lampholder': fields.many2one('ledia.product.lampholder','Casquillo'),
        'prop_format': fields.many2one('ledia.product.format','Formato'),
        'prop_power': fields.integer('Potencia (W)'),
        'prop_power_led': fields.integer('Potencia LED (W)'),
        'prop_power_replace': fields.integer('Reemplazo (W)'),
        'prop_power_saving': fields.function(calc_power_saving, method=True, type='integer', string='Ahorro (%)'),
        'prop_power_factor': fields.integer('Factor'),
        'prop_cri': fields.integer('CRI (%)', help="Índice de rendimiento del color"),
        'prop_luminous_efficacy': fields.function(calc_luminous_efficacy, method=True, type='integer', string='Eficacia (lm/W)'),
        'prop_material_type': fields.char('Material',size=255),
        'prop_total_led': fields.integer('Número de LED'),
        'prop_luminous_flux': fields.integer('Flujo (lm)'),
        'prop_luminous_flux_tolerance': fields.integer('Tolerancia del flujo'),
        'prop_angle_lamp': fields.integer('Angulo de lamp.'),
        'prop_application': fields.many2many('ledia.product.application','product_product2ledia_product_application','product_id','application_id','Aplicaciones'),
        'prop_cert': fields.many2many('ledia.product.cert','product_product2ledia_product_cert','product_id','cert_id','Certificados'),
        'prop_length': fields.integer('Largo (mm)'),
        'prop_width': fields.integer('Ancho (mm)'),
        'prop_height': fields.integer('Alto (mm)'),
    }
    _defaults = {
    }
product_product()
